#pragma once

#include <string.h>

/*
 * State machine base implementation
 */
template<typename TState, typename TBase>
class CState_Machine {
    public:
        using TState_Transition_Cb = void(TBase::*)(TState src, TState dst);

    private:
        static constexpr int s2i(TState s) {
            return static_cast<int>(s);
        }
        static constexpr int State_Count = s2i(TState::count);

    private:
        TState mCurrent_State;
        TState_Transition_Cb mTransition_Matrix[State_Count*State_Count];
        TBase* mBase;

    public:
        CState_Machine(TState initialState, TBase* base) : mCurrent_State(initialState), mBase(base) {
            memset(mTransition_Matrix, 0, sizeof(mTransition_Matrix));
        }

        void Add_Transition(TState src, TState dst, TState_Transition_Cb cb) {
            mTransition_Matrix[s2i(src) + State_Count*s2i(dst)] = cb;
        }

        void Remove_Transition(TState src, TState dst) {
            mTransition_Matrix[s2i(src) + State_Count*s2i(dst)] = nullptr;
        }

        bool Move(TState dst) {

            TState_Transition_Cb cb = mTransition_Matrix[s2i(mCurrent_State) + State_Count*s2i(dst)];

            if (cb == nullptr) {
                return false;
            }

            (*mBase.*cb)(mCurrent_State, dst);

            mCurrent_State = dst;

            return true;
        }

        TState Get_Current_State() const {
            return mCurrent_State;
        }
};
