#include "linkproto.h"

#include "freertos/timers.h"
#include "esp_log.h"

#define TAG "LINKPROTO"

CLink_Proto_Mgr* CLink_Proto_Mgr::gInstance;

static constexpr int EVENT_STATE_TRANSITION = BIT0;
static constexpr int EVENT_DATA_FROM_L1 = BIT1;
static constexpr int EVENT_DATA_FROM_L3 = BIT2;
static constexpr int EVENT_INTERNAL = BIT3;
static constexpr int EVENT_EXTERNAL = BIT4;

static constexpr int EVENT_BITS_ALL = EVENT_STATE_TRANSITION | EVENT_DATA_FROM_L1 | EVENT_DATA_FROM_L3 | EVENT_INTERNAL | EVENT_EXTERNAL;

CLink_Proto_Mgr::CLink_Proto_Mgr() : mState(NLink_State::None, this) {

    gInstance = this;

    mEvent_Handle = xEventGroupCreate();

    mState.Add_Transition(NLink_State::None, NLink_State::Discovery, &CLink_Proto_Mgr::Transition);
    mState.Add_Transition(NLink_State::Discovery, NLink_State::Operational, &CLink_Proto_Mgr::Transition);

    // TODO: initialize ESP-NOW, which in turn creates transition event to Discovery phase
}

void CLink_Proto_Mgr::Transition(NLink_State from, NLink_State to) {
    ESP_LOGI(TAG, "Transition from %d to %d", (int)from, (int)to);

    xEventGroupSetBits(mEvent_Handle, EVENT_STATE_TRANSITION);
}

void CLink_Proto_Mgr::Task_Run() {

    ESP_LOGI(TAG, "Task running");

    while (true) {

        EventBits_t bits = xEventGroupWaitBits(mEvent_Handle, EVENT_BITS_ALL, pdFALSE, pdFALSE, portMAX_DELAY);

        ESP_LOGI(TAG, "Triggered event bitmask: %d", (int)bits);

        if (bits & EVENT_STATE_TRANSITION) {
            //
        }

        if (bits & EVENT_DATA_FROM_L1) {
            //
        }

        if (bits & EVENT_DATA_FROM_L3) {
            //
        }

        if (bits & EVENT_INTERNAL) {
            //
        }

        if (bits & EVENT_EXTERNAL) {
            //
        }

        xEventGroupClearBits(mEvent_Handle, 0xFF);
    }
}
