#include "peertable.h"
#include "esp_log.h"
#include "esp_mac.h"

#include <cstring>

#define TAG "PeerTable"

namespace {
    TPeer_Table_Entry Peer_Table[Peer_Max_Count];
}

namespace peers {

    void Table_Init() {
        for (int i = 0; i < Peer_Max_Count; i++) {
            Peer_Table[i].valid = false;
        }
        ESP_LOGI(TAG, "Peer table initialized");
    }

    const TPeer_Table_Entry* Find_Peer_By_Link_Address(uint8_t link_addr[ESP_NOW_ETH_ALEN]) {
        for (int i = 0; i < Peer_Max_Count; i++) {
            if (Peer_Table[i].valid) {
                if (memcmp(Peer_Table[i].link_addr, link_addr, ESP_NOW_ETH_ALEN) == 0) {
                    return &Peer_Table[i];
                }
            }
        }

        return nullptr;
    }

    const TPeer_Table_Entry* Find_Peer_By_Net_Address(uint16_t net_addr) {
        for (int i = 0; i < Peer_Max_Count; i++) {
            if (Peer_Table[i].valid) {
                if (Peer_Table[i].net_addr == net_addr) {
                    return &Peer_Table[i];
                }
            }
        }

        return nullptr;
    }

    bool Add_Peer(uint8_t link_addr[ESP_NOW_ETH_ALEN]) {

        for (int i = 0; i < Peer_Max_Count; i++) {
            if (!Peer_Table[i].valid) {
                memcpy(Peer_Table[i].link_addr, link_addr, ESP_NOW_ETH_ALEN);
                Peer_Table[i].net_addr = Net_Addr_None;
                Peer_Table[i].valid = true;
                ESP_LOGI(TAG, "Added peer " MACSTR, MAC2STR(link_addr));
                return true;
            }
        }

        return false;
    }

    bool Remove_Peer(uint8_t link_addr[ESP_NOW_ETH_ALEN]) {
        for (int i = 0; i < Peer_Max_Count; i++) {
            if (Peer_Table[i].valid) {
                if (memcmp(Peer_Table[i].link_addr, link_addr, ESP_NOW_ETH_ALEN) == 0) {
                    Peer_Table[i].valid = false;
                    ESP_LOGI(TAG, "Removed peer " MACSTR, MAC2STR(link_addr));
                    return true;
                }
            }
        }

        return false;
    }

    bool Set_Peer_Net_Address(uint8_t link_addr[ESP_NOW_ETH_ALEN], uint16_t net_addr) {
        for (int i = 0; i < Peer_Max_Count; i++) {
            if (Peer_Table[i].valid) {
                if (memcmp(Peer_Table[i].link_addr, link_addr, ESP_NOW_ETH_ALEN) == 0) {
                    Peer_Table[i].net_addr = net_addr;
                    ESP_LOGI(TAG, "Peer " MACSTR " assigned net address %d", MAC2STR(link_addr), (int)net_addr);
                    return true;
                }
            }
        }

        return false;
    }
}
