#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <assert.h>
#include "freertos/FreeRTOS.h"
#include "freertos/semphr.h"
#include "freertos/timers.h"
#include "nvs_flash.h"
#include "esp_random.h"
#include "esp_event.h"
#include "esp_netif.h"
#include "esp_wifi.h"
#include "esp_log.h"
#include "esp_mac.h"
#include "esp_now.h"
#include "esp_crc.h"

#include "linkproto/linkproto.h"
#include "netproto/netproto.h"
#include "linkproto/peertable.h"

#include "sensors/sensors.h"

#define TAG "BSS-Main"

extern "C" void linkproto_task(void*) {
    CLink_Proto_Mgr mgr;

    mgr.Task_Run();
}

extern "C" void netproto_task(void*) {
    CNet_Proto_Mgr mgr;

    mgr.Task_Run();
}

extern "C" void sensor_task(void*) {
    CSensor_Mgr mgr;

    mgr.Task_Run();
}

extern "C" void app_main(void)
{
    // Initialize NVS
    esp_err_t ret = nvs_flash_init();
    if (ret == ESP_ERR_NVS_NO_FREE_PAGES || ret == ESP_ERR_NVS_NEW_VERSION_FOUND) {
        ESP_ERROR_CHECK( nvs_flash_erase() );
        ret = nvs_flash_init();
    }

    peers::Table_Init();

    ESP_LOGI(TAG, "Starting tasks");

    xTaskCreate(&linkproto_task, "linkproto_task", 4096, NULL, 10, NULL);
    xTaskCreate(&netproto_task, "netproto_task", 4096, NULL, 10, NULL);
    xTaskCreate(&sensor_task, "sensor_task", 4096, NULL, 10, NULL);
}
