#include "netproto.h"

#include "freertos/timers.h"
#include "esp_log.h"

#define TAG "NETPROTO"

CNet_Proto_Mgr* CNet_Proto_Mgr::gInstance;

static constexpr int EVENT_STATE_TRANSITION = BIT0;
static constexpr int EVENT_DATA_FROM_L2 = BIT1;
static constexpr int EVENT_DATA_FROM_L4 = BIT2;
static constexpr int EVENT_INTERNAL = BIT3;
static constexpr int EVENT_EXTERNAL = BIT4;

static constexpr int EVENT_BITS_ALL = EVENT_STATE_TRANSITION | EVENT_DATA_FROM_L2 | EVENT_DATA_FROM_L4 | EVENT_INTERNAL | EVENT_EXTERNAL;

CNet_Proto_Mgr::CNet_Proto_Mgr() : mState(NNet_State::None, this) {

    gInstance = this;

    mEvent_Handle = xEventGroupCreate();

    mState.Add_Transition(NNet_State::None, NNet_State::Address_Negotiation, &CNet_Proto_Mgr::Transition);
    mState.Add_Transition(NNet_State::Address_Negotiation, NNet_State::Topology_Discovery, &CNet_Proto_Mgr::Transition);
    mState.Add_Transition(NNet_State::Topology_Discovery, NNet_State::Operational, &CNet_Proto_Mgr::Transition);
}

void CNet_Proto_Mgr::Transition(NNet_State from, NNet_State to) {
    //

    xEventGroupSetBits(mEvent_Handle, EVENT_STATE_TRANSITION);
}

void CNet_Proto_Mgr::Task_Run() {

    ESP_LOGI(TAG, "Task running");

    while (true) {

        EventBits_t bits = xEventGroupWaitBits(mEvent_Handle, EVENT_BITS_ALL, pdFALSE, pdFALSE, portMAX_DELAY);

        ESP_LOGI(TAG, "Triggered event bitmask: %d", (int)bits);

        if (bits & EVENT_STATE_TRANSITION) {
            //
        }

        if (bits & EVENT_DATA_FROM_L2) {
            //
        }

        if (bits & EVENT_DATA_FROM_L4) {
            //
        }

        if (bits & EVENT_INTERNAL) {
            //
        }

        if (bits & EVENT_EXTERNAL) {
            //
        }

        xEventGroupClearBits(mEvent_Handle, 0xFF);
    }
}
