#ifndef BSS_LINKPROTO_H
#define BSS_LINKPROTO_H

#include "freertos/FreeRTOS.h"

#include "../common/statemachine.h"
#include "../common/taskrunnable.h"
#include "freertos/event_groups.h"
#include "freertos/queue.h"

#include "espnow_layer.h"
#include "peertable.h"

constexpr size_t Max_L2_Frame_Size = 128;

enum class NLink_State {
    None,
    Discovery,
    Operational,

    count
};

#pragma pack(push, 1)

struct CRaw_Data {
    uint8_t src_mac[ESP_NOW_ETH_ALEN];
    uint8_t dst_mac[ESP_NOW_ETH_ALEN];
    int rssi;
    uint16_t data_len;
    uint8_t data[0];    // just a data break
};

#pragma pack(pop)

namespace l2pdu {

    enum class NOpcode : uint8_t {
        NONE,
        HELLO,
        HANDSHAKE,
        HANDSHAKE_REPLY,
        L3_PROTO_DATA,
    };

#pragma pack(push, 1)

    struct TFrame_Header {
        NOpcode opcode;
    };

    struct THello {
        TFrame_Header header;
        uint8_t magic_number;
    };

    struct TPeer_Handshake {
        TFrame_Header header;
        uint8_t peer_count;
    };

    struct TPeer_Handshake_Reply {
        TFrame_Header header;
        uint8_t peer_count;
    };

    struct TL3_Data {
        TFrame_Header header;
        uint16_t data_len;
        uint8_t data[0];
    };

#pragma pack(pop)
}

class CLink_Proto_Mgr : public CTask_Runnable {
    private:
        CESP_NOW mESP_NOW;
        CState_Machine<NLink_State, CLink_Proto_Mgr> mState;
        EventGroupHandle_t mEvent_Handle;
        QueueHandle_t mIncoming_Data_Queue;
        QueueHandle_t mOutgoing_Data_Queue;

        static CLink_Proto_Mgr* gInstance;

    protected:
        void Transition(NLink_State from, NLink_State to);

    protected:
        // station broadcasts hello frame to all sorrounding nodes
        uint8_t* Build_Hello_Frame(uint8_t& size);
        // a node hears Hello frame and tries to shake hands with us
        uint8_t* Build_Peer_Handshake_Frame(uint8_t& size);
        // we hear the handshake and acknowledge the neighborhood
        uint8_t* Build_Peer_Handshake_Reply_Frame(uint8_t& size);
        // we transfer the L3 data
        uint8_t* Build_L3_Data_Frame(uint8_t& size, uint8_t* l3_data, uint8_t l3_data_len);

        bool Process_L2_Frame(CRaw_Data* raw);

    public:
        CLink_Proto_Mgr();

        void Task_Run() override;
        void Incoming_Data(CRaw_Data* data);

        void Send_L3_Data(const uint8_t* dst_mac, uint8_t* data, uint8_t data_len);

        static CLink_Proto_Mgr* Instance() {
            return gInstance;
        }
};

#endif
