#include <iostream>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <cstring>

#include <cstdint>

#include <chrono>
#include <ctime>
#include <iomanip>
#include <sstream>

#define UDP_ADDRESS "192.168.69.1"
#define UDP_PORT 8266
#define UDP_REPLY_OK "OK"
#define UDP_REPLY_ERROR "ERROR"

/**
 * @brief An enumeration of all possible measurement types
 *
 */
enum class measurement_type : uint8_t {
    temperature = 1,
    humidity = 2,
    ambient_light = 3,
    sound_level = 4,
    // --- HERE YOU CAN ADD MORE MEASUREMENTS ---
};

/**
 * @brief Binds the UDP port, parses the request
 *
 * @return int
 */
int main() {

    int sockfd;
    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket failed");
        return EXIT_FAILURE;
    }

    struct sockaddr_in addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(UDP_PORT);
    addr.sin_addr.s_addr = inet_addr(UDP_ADDRESS); // INADDR_ANY can also be used, but to separate WSN from Internet, this is better
    memset(addr.sin_zero, 0, sizeof(addr.sin_zero));

    if ((bind(sockfd, (struct sockaddr*)&addr, sizeof(addr))) < 0) {
        perror("bind failed");
        return EXIT_FAILURE;
    }

    char buffer[1024]; // buffer for messages
    struct sockaddr_in sender_addr;
    socklen_t sender_addrlen = sizeof(sender_addr);

    while (true) {
        // incoming UDP datagram
        int num_bytes = recvfrom(sockfd, buffer, sizeof(buffer), 0, (struct sockaddr*)&sender_addr, &sender_addrlen);
        if (num_bytes < 0) {
            perror("recvfrom failed");
            break;
        }
        buffer[num_bytes] = '\0';
        std::cout << "Received " << num_bytes << " bytes from " << inet_ntoa(sender_addr.sin_addr) << ":" << ntohs(sender_addr.sin_port) << std::endl;

        // parse Protobuf zpravy
        std::cout << "Processing... ";

        for (size_t i = 0; i < num_bytes; i += 3) {
            measurement_type mtype = static_cast<measurement_type>(buffer[i]);
            int16_t value = *reinterpret_cast<int16_t*>(&buffer[i+1]);
            
            std::cout << "Measurement of " << static_cast<int>(mtype) << ", value = " << value << std::endl;
            
            // TODO: put it into a datagram and send to the cloud
        }

        std::cout << "Done." << std::endl;

        std::cout << "Success!" << std::endl;
        sendto(sockfd, UDP_REPLY_OK, strlen(UDP_REPLY_OK), 0, (struct sockaddr*)&sender_addr, sender_addrlen);
    }

    close(sockfd);
    return EXIT_SUCCESS;
}


