#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <espnow.h>

// we save this to the RTC memory
struct {
  uint32_t crc32;
  uint8_t channel;
  uint8_t ap_mac[6];
} rtcData;

void setup() {
  Serial.begin(9600);
  Serial.println("Start!");

  String resetCause = ESP.getResetReason();
  Serial.print("The SoC reset due to: ");
  Serial.println(resetCause);

  // External System
  // Deep-Sleep Wake
}

void startWiFi() {

  Serial.println("Starting WiFi...");

  // for time measurement
  long msStart = millis();

  WiFi.mode(WIFI_OFF);
  WiFi.forceSleepBegin();
  delay(1);

  // read from RTC, check for valid signature
  bool rtcValid = false;
  if (ESP.rtcUserMemoryRead(0, (uint32_t*)&rtcData, sizeof(rtcData))) {
    uint32_t crc = crc32(((uint8_t*)&rtcData) + 4, sizeof(rtcData) - 4);
    if (crc == rtcData.crc32) {
      rtcValid = true;
    }
  }

  WiFi.forceSleepWake();

  // do not save WiFi state to flash; station mode; output power reduced
  WiFi.persistent(false);
  WiFi.mode(WIFI_STA);
  WiFi.setOutputPower(10);
  // check valid? connect using known data
  if (rtcValid)
    WiFi.begin("KIV-BSS-gateway", "KIVbss4243", rtcData.channel, rtcData.ap_mac, true);
  else
    WiFi.begin("KIV-BSS-gateway", "KIVbss4243");

  // while not connected, wait
  while (WiFi.status() != WL_CONNECTED) {
    yield();  // this is due to the software watchdog firing up reset
  }

  // save obtained info to RTC memory
  rtcData.channel = WiFi.channel();
  memcpy(rtcData.ap_mac, WiFi.BSSID(), 6);
  rtcData.crc32 = crc32(((uint8_t*)&rtcData) + 4, sizeof(rtcData) - 4);
  ESP.rtcUserMemoryWrite(0, (uint32_t*)&rtcData, sizeof(rtcData));

  // print info and time measurement
  Serial.print("Connected in ");
  Serial.print((millis() - msStart));
  Serial.println("ms");

  WiFi.setAutoReconnect(true);
}

void loop() {

  // connect to WiFi
  startWiFi();

  Serial.println("Wait before sleep");

  // wait some time so we can measure the current
  delay(5000);

  Serial.println("Putting WiFi to sleep...");

  WiFi.mode(WIFI_OFF);
  WiFi.forceSleepBegin();

  delay(5000);

  Serial.println("Going to sleep...");

  // WAKE_RF_DEFAULT
  // WAKE_RFCAL
  // WAKE_NO_RFCAL
  // WAKE_RF_DISABLED

  // deep sleep for 10 seconds
  ESP.deepSleep(10e6, WAKE_RF_DISABLED);

  Serial.println("We should never be there!");
}