#include "sensors.h"

#include "freertos/FreeRTOS.h"
#include "freertos/timers.h"
#include "esp_log.h"
#include "esp_random.h"

#include "../netproto/netproto.h"
#include "units/units.h"

#include "mbedtls/aes.h"

#define TAG "SENSORS"

uint8_t key[32] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1};
uint8_t iv[16];

void encrypt(uint8_t* data, uint16_t len, uint8_t** data_out, uint16_t& out_len) {
    memset(iv, 0, sizeof(iv));

    mbedtls_aes_context aes; //aes is simple variable of given type
    mbedtls_aes_init(&aes);
    mbedtls_aes_setkey_enc(&aes, key, 256);

    *data_out = (uint8_t*)malloc(16);

    mbedtls_aes_crypt_cbc(&aes, MBEDTLS_AES_ENCRYPT, len, iv, (unsigned char *)data, *data_out);
}

CSensor_Mgr* CSensor_Mgr::gInstance = nullptr;

CSensor_Mgr::CSensor_Mgr() {
    gInstance = this;
}

void CSensor_Mgr::Incoming_Data(uint8_t* data, uint16_t data_len) {
    ESP_LOGI(TAG, "Incoming data!");

    for (int i = 0; i < data_len; i++) {
        esp_log_write(ESP_LOG_INFO, TAG, "%02x ", data[i]);
    }
    esp_log_write(ESP_LOG_INFO, TAG, "\n");
}

void CSensor_Mgr::Send_Aggregate_Data(uint8_t addr) {

    const int cnt = (int)sensors::Measurement_Type::count;
    const uint16_t size = sizeof(app_pdu::TSensor_Data) * cnt;

    app_pdu::TSensor_Data* data = (app_pdu::TSensor_Data*)malloc(size);

    for (int i = 0; i < cnt; i++) {
        data[i].sensor = i+1;
        data[i].value = sensors::Get_Measurement((sensors::Measurement_Type)(i+1));
    }

    CNet_Proto_Mgr::Instance()->Send_Data(addr, (uint8_t*)data, size);
}

void CSensor_Mgr::Task_Run() {

    ESP_LOGI(TAG, "Task running");

    while (true) {
        vTaskDelay((10000 + (esp_random() % 1000))/portTICK_PERIOD_MS);

        auto* net = CNet_Proto_Mgr::Instance();

        if (net->Is_Ready()) {

            if (net->Get_My_Address() == 1) {
                Send_Aggregate_Data(2);
            }
            else {
                Send_Aggregate_Data(1);
            }

        }
    }

}
